package jdh.demos;

import java.io.*;
import java.net.InetAddress;
import java.util.*;

import net.jxta.discovery.*;
import net.jxta.document.*;
import net.jxta.endpoint.InputStreamMessageElement;
import net.jxta.endpoint.MessageElement;
import net.jxta.id.IDFactory;
import net.jxta.peergroup.*;
import net.jxta.pipe.*;
import net.jxta.protocol.*;
import org.jabber.jabberbeans.*;
import org.jabber.jabberbeans.Extension.IQAuthBuilder;
import org.jabber.jabberbeans.util.*;

public class JxtaClient
  implements DiscoveryListener, PacketListener, PipeMsgListener {

  private ConnectionBean cb;
  private String username, password, resource, server_host;
  private RosterBean rosterBean;
  private JID myJID;

  private PeerGroup rootGroup;
  private Vector msgQueue = new Vector();

  public JxtaClient(String[] args) {
    username = args[0];
    password = args[1];
    server_host = args[2];
    resource = args[3];
    try {
      startJabber();
      startJxta();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private void startJxta() throws Exception {
    rootGroup = PeerGroupFactory.newNetPeerGroup();

    DiscoveryService disco = rootGroup.getDiscoveryService();
    disco.addDiscoveryListener(this);

    disco.flushAdvertisements(null, DiscoveryService.GROUP);
    disco.flushAdvertisements(null, DiscoveryService.PEER);
    disco.flushAdvertisements(null, DiscoveryService.ADV);

    PipeAdvertisement pipeadv;
    Enumeration pipeAds =
      disco.getLocalAdvertisements(
        DiscoveryService.ADV,
        "name",
        "JxtaTalkUserName." + username);
    if (pipeAds.hasMoreElements()) {
      pipeadv = (PipeAdvertisement) pipeAds.nextElement();
    } else {
      pipeadv =
        (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
          PipeAdvertisement.getAdvertisementType());
      PeerGroupID pgid = rootGroup.getPeerGroupID();
      pipeadv.setPipeID(IDFactory.newPipeID(pgid));
      pipeadv.setName("JxtaTalkUserName." + username);
      pipeadv.setType(PipeService.UnicastType);
    }
    InputPipe inputPipe =
      rootGroup.getPipeService().createInputPipe(pipeadv, this);
    disco.publish(pipeadv, DiscoveryService.ADV);
    disco.remotePublish(pipeadv, DiscoveryService.ADV);
  }

  private void startJabber() throws Exception {
    cb = new ConnectionBean();
    SyncPacketListener sync = new SyncPacketListener(cb);
    Packet p;

    myJID = new JID(username, password, resource);

    cb.addPacketListener(new PacketDebug());
    cb.connect(InetAddress.getByName(server_host));
    InfoQueryBuilder iqb = new InfoQueryBuilder();
    IQAuthBuilder auth = new IQAuthBuilder();
    iqb.setType("set");
    auth.setUsername(username);
    auth.setPassword(password);
    auth.setResource(resource);

    iqb.addExtension(auth.build());
    Packet iq = iqb.build();
    p = sync.sendAndWait((ContentPacket) iq, 5000);

    new MessengerBean(cb).addPacketListener(this);
    rosterBean = new RosterBean();
    rosterBean.setIQBean(new IQBean(cb));
    rosterBean.refreshRoster();
    PresenceBean presenceBean = new PresenceBean();
    presenceBean.setConnBean(cb);
    presenceBean.addPresenceListener(new OKPresenceListener());
    PresenceBuilder pb = new PresenceBuilder();
    pb.setStatus("Available");
    presenceBean.getConnBean().send(pb.build());
  }

  public void receivedPacket(PacketEvent pe) {
    Message jabberMsg = (Message) pe.getPacket();
    try {
      if ((jabberMsg.getSubject() == null)
        || jabberMsg.getSubject().equals("")) {
        // This is a 'discover' 'peers' or 'groups' message
        String command = jabberMsg.getBody().toLowerCase();
        if (command.startsWith("discover")) {
          System.out.println("Discovering....");
          rootGroup.getDiscoveryService().getRemoteAdvertisements(
            null,
            DiscoveryService.GROUP,
            null,
            null,
            10);
          rootGroup.getDiscoveryService().getRemoteAdvertisements(
            null,
            DiscoveryService.PEER,
            null,
            null,
            10);
        } else if (command.startsWith("peers")) {
          System.out.println("listing peers....");
          Enumeration ads =
            rootGroup.getDiscoveryService().getLocalAdvertisements(
              DiscoveryService.PEER,
              null,
              null);
          StringBuffer response = new StringBuffer(1024);
          response.append("Known Peers are: ");
          while (ads.hasMoreElements()) {
            PeerAdvertisement peerAd =
              (PeerAdvertisement) ads.nextElement();
            response.append(peerAd.getName());
            if (ads.hasMoreElements())
              response.append(", ");
          }
          MessageBuilder mb = new MessageBuilder();
          mb.setBody(response.toString());
          mb.setSubject(jabberMsg.getSubject());
          mb.setThread(jabberMsg.getThread());
          mb.setFromAddress(jabberMsg.getToAddress());
          mb.setType(jabberMsg.getType());
          mb.setToAddress(jabberMsg.getFromAddress());
          cb.send(mb.build());

        } else if (command.startsWith("groups")) {
          System.out.println("listing groups....");
          Enumeration ads =
            rootGroup.getDiscoveryService().getLocalAdvertisements(
              DiscoveryService.GROUP,
              null,
              null);
          StringBuffer response = new StringBuffer(1024);
          response.append("Known Groups are: ");
          while (ads.hasMoreElements()) {
            PeerGroupAdvertisement peerGroupAd =
              (PeerGroupAdvertisement) ads.nextElement();
            response.append(peerGroupAd.getName());
            if (ads.hasMoreElements())
              response.append(", ");
          }
          MessageBuilder mb = new MessageBuilder();
          mb.setBody(response.toString());
          mb.setSubject(jabberMsg.getSubject());
          mb.setThread(jabberMsg.getThread());
          mb.setFromAddress(jabberMsg.getToAddress());
          mb.setType(jabberMsg.getType());
          mb.setToAddress(jabberMsg.getFromAddress());
          cb.send(mb.build());
        }
      } else {
        /* Assume it's a message for a JXTA peer, 
         * Subject contains the dest peer name */
        Enumeration ads =
          rootGroup.getDiscoveryService().getLocalAdvertisements(
            DiscoveryService.ADV,
            "Name",
            "JxtaTalkUserName." + jabberMsg.getSubject());
        if (ads.hasMoreElements()) {
          PipeAdvertisement ad = (PipeAdvertisement) ads.nextElement();
          sendToPipe(ad, jabberMsg);
        } else { // can't do it now. Queue it.
          synchronized (this) {
            msgQueue.add(jabberMsg);
          }
          rootGroup.getDiscoveryService().getRemoteAdvertisements(
            null,
            DiscoveryService.ADV,
            "Name",
            "JxtaTalkUserName." + jabberMsg.getSubject(),
            10);
        }
      }
    } catch (Exception e) {}
  }

  public void discoveryEvent(DiscoveryEvent event) {
    String type = "UNKNOWN";
    switch (event.getResponse().getDiscoveryType()) {
      case DiscoveryService.ADV :
        type = "ADV";
        // We may have discovered a talk pipe advert.
        // Send any messages that are waiting.
        checkPendingMessages(event.getSearchResults());
        break;
      case DiscoveryService.GROUP :
        type = "GROUP";
        break;
      case DiscoveryService.PEER :
        type = "PEER";
        break;
    }
    Enumeration results = event.getSearchResults();
    while (results.hasMoreElements()) {
      Advertisement adv = (Advertisement) results.nextElement();
      sendToAll(type + " Advertisement Discovered", adv.toString());
    }

  }

  private void sendToAll(String subject, String msgText) {
    Enumeration roster = rosterBean.entries();
    while (roster.hasMoreElements()) {
      RosterItem item = (RosterItem) roster.nextElement();
      MessageBuilder mb = new MessageBuilder();
      mb.setToAddress(item.getJID());
      mb.setFromAddress(myJID);
      mb.setSubject(subject);
      mb.setBody(msgText);
      try {
        cb.send(mb.build());
      } catch (InstantiationException e) {}
    }
  }

  private synchronized void checkPendingMessages(Enumeration ads) {
    if (msgQueue.isEmpty())
      return;
    Vector sentMessages = new Vector();
    while (ads.hasMoreElements()) {
      Advertisement ad = (Advertisement) ads.nextElement();
      if (ad instanceof PipeAdvertisement) {
        PipeAdvertisement newPipeAd = (PipeAdvertisement) ad;
        String name = newPipeAd.getName();
        Enumeration waitingMessages = msgQueue.elements();
        while (waitingMessages.hasMoreElements()) {
          Message jabberMsg = (Message) waitingMessages.nextElement();
          if (name.equals("JxtaTalkUserName." + jabberMsg.getSubject())) {
            sendToPipe(newPipeAd, jabberMsg);
            sentMessages.add(jabberMsg);
          }
        }
      }
    }
    msgQueue.removeAll(sentMessages);
  }

  public void pipeMsgEvent(PipeMsgEvent event) {
    net.jxta.endpoint.Message msg = event.getMessage();
    MessageElement nameElement =
      msg.getMessageElement("JxtaTalkSenderName");
    MessageElement msgElement =
      msg.getMessageElement("JxtaTalkSenderMessage");
    String subject =
      "Jxta Talk Message From " + new String(nameElement.getBytes(false));
    String message = new String(msgElement.getBytes(false));
    sendToAll(subject, message);
  }

  private void sendToPipe(PipeAdvertisement pipeAd, Message jabberMsg) {
    try {
      OutputPipe outputPipe =
        rootGroup.getPipeService().createOutputPipe(pipeAd, 1000);
      net.jxta.endpoint.Message jxtaMsg = new net.jxta.endpoint.Message();
      MessageElement messageElement = null;

      messageElement =
        new InputStreamMessageElement(
          "JxtaTalkSenderName",
          new MimeMediaType("text/plain"),
          new ByteArrayInputStream(username.getBytes()),
          null);

      jxtaMsg.replaceMessageElement(messageElement);
      messageElement =
        new InputStreamMessageElement(
          "JxtaTalkSenderMessage",
          new MimeMediaType("text/plain"),
          new ByteArrayInputStream(jabberMsg.getBody().getBytes()),
          null);

      jxtaMsg.replaceMessageElement(messageElement);

      outputPipe.send(jxtaMsg);
      outputPipe.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  // Unneeded Jabberbeans PacketListener methods
  public void sendFailed(PacketEvent pe) {}
  public void sentPacket(PacketEvent pe) {}

  // Utility class to acknowledge presence subscribe/unsubscribe requests
  private class OKPresenceListener implements PresenceListener {

    public void subscribe(Presence p) {
      reply(p, "subscribed");
    }
    public void unsubscribe(Presence p) {
      reply(p, "unsubscribed");
    }

    // Unneeded Jabberbeans PresenceListener methods
    public void changedPresence(
      Hashtable t,
      Presence p,
      PresenceUserNode n,
      String s) {
      }
    public void error(Presence p) {}
    public void subscribed(Presence p) {}
    public void unsubscribed(Presence p) {}

    private void reply(Presence p, String message) {
      try {
        PresenceBuilder pb = new PresenceBuilder();
        pb.setFromAddress(p.getToAddress());
        pb.setToAddress(p.getFromAddress());
        pb.setType(message);
        cb.send(pb.build());
      } catch (InstantiationException e) {
        e.printStackTrace();
      }
    }
  }

  public static void main(String[] args) {
    new JxtaClient(args);
  }
}
