package require jabberlib
package require Tclx

proc make_gui {} {
	global b
	global jid 
	global pass 
	global otherJid
	global chatEntry
	global output

	set jid "user1@my-jabber/chat"
	set pass "user1"
	set otherJid "user2@my-jabber"
	set chatEntry ""

  # Create a root window with three partitions in it
	wm title . "Jabber Chat"

  frame .button_frame 
	pack .button_frame -side bottom
  frame .bframe 
	pack .bframe -side bottom
  frame .tframe
	pack .tframe -side top

  # The bottom frame gets a button and three text boxes
	frame .button_frame.lframe 
	pack .button_frame.lframe -side left
	label .button_frame.lframe.lab -text "Your  JID"
	pack .button_frame.lframe.lab -side left
	entry .button_frame.lframe.jid_entry -textvariable jid
	pack .button_frame.lframe.jid_entry -side right

	frame .button_frame.lframe2
	pack .button_frame.lframe2 -side left
	label .button_frame.lframe2.lab -text "Password "
	pack .button_frame.lframe2.lab -side left
	entry .button_frame.lframe2.pass_entry -textvariable pass
	pack .button_frame.lframe2.pass_entry -side right

	frame .button_frame.lframe3
	pack .button_frame.lframe3 -side left
	label .button_frame.lframe3.lab -text "Other JID"
	pack .button_frame.lframe3.lab -side left
	entry .button_frame.lframe3.pass_entry -textvariable otherJid
	pack .button_frame.lframe3.pass_entry -side right

	button .button_frame.b -text "Connect" -command buttonPress
	set b .button_frame.b
	pack .button_frame.b -side right

  # The top frame gets a multi-line scrolled text window to show messages
  text .tframe.text -width 80 -height 20 -yscrollcommand ".tframe.scroll set"
	scrollbar .tframe.scroll -command ".tframe.text yview"
	pack .tframe.scroll -fill y -side left
	pack .tframe.text -side top -fill both -anchor n -expand true
	set output .tframe.text

	.tframe.text tag configure mine -foreground red
	.tframe.text tag configure his  -foreground blue
  
  # The bottom frame gets a single-line text window for message entry
	entry .bframe.text -width 80 -textvariable chatEntry
	pack .bframe.text -side bottom -fill both -anchor s

  bind .bframe.text <Return> sendMsg
}

proc buttonPress {} {
	global b
	global jid
	global pass
	global otherJid
  set curLabel [$b cget -text]
  # A little cheesy. The first time the button is pushed, it 
  # connects.  All subsequent times, it sends the presence.
  if {[string compare $curLabel "Connect"] == 0} {
    connectToJabber $jid $pass $otherJid
    $b configure -text "Send\nPresence"
	} else {
    sendPresence
	}
}

proc connectToJabber {myjid passwd chatjid} {
	global output
	global auth_result

	set start_idx [expr [string first "@" $myjid] + 1]
	set end_idx [expr [string first "/" $myjid] - 1]
	set server [string range $myjid $start_idx $end_idx]
	set idx [expr [string first "@" $myjid] - 1]
	set user [string range $myjid 0 $idx]
  set sock [socket $server 5222]

  jlib::connect $sock $server

  jlib::send_auth $user $passwd aresource recv_auth_result
  vwait auth_result

  if {$auth_result == "OK"} {
	  $output insert end "Connected\n"
	}

	jlib::send_presence -stat Online 
	jlib::roster_get -command roster_cmd
	jlib::send_presence
}

proc sendMsg {} {
	global chatEntry
	global output
	global jid
	global otherJid
  set text  $chatEntry
	set idx [expr [string first "@" $jid] - 1]
	set uname [string range $jid 0 $idx]
	$output insert end "<$uname> $text\n" mine
	jlib::send_msg $otherJid -type chat -thread chat_demo_thread \
      -subject chat_demo_subject -body $text
  set chatEntry ""
}


proc sendPresence {} {
	jlib::send_presence -show chat -stat {Let's Roll}
}

proc client:presence {from type x args} {
  switch -- $type {
    subscribe {
		  jlib::send_presence -to $from -type subscribed
		  jlib::send_presence -to $from -type subscribe
		}
    unsubscribe {
		  jlib::send_presence -to $from -type unsubscribed
		  jlib::send_presence -to $from -type unsubscribe
		}
    default {
      global output
			set show ""
			set status ""
		  foreach {attr val} $args {	
			  if {[string match -show $attr]} {set show $val}
			  if {[string match -status $attr]} {set status $val}
      }
			if {[string length $show] == 0} {set show "normal"}
			if {[string length $status] == 0} {set status "Online"}

			$output insert end "<roster> $from is $show/$status\n"
		}
	}
}


proc client:message {from id type subject body err thread priority x} {
	global output
  echo chat "MESSAGE: $from; $id $type; $subject; $body; $err;\
    $thread; $priority; $x"
	set idx [expr [string first "@" $from] - 1]
	set uname [string range $from 0 $idx]
	$output insert end "<$uname> $body\n" his
}

proc roster_cmd {status} {
	#status is BEGIN_ROSTER or END_ROSTER
	# END_ROSTER means the roster is ready to use
	global output
	echo "Roster CMD: $status\n"
}

proc client:roster_item {jid name groups subsc ask category subtype} {
    echo ROSTER_ITEM $jid $name $groups $subsc $ask $category $subtype
}

proc client:roster_push {jid name groups subsc ask category subtype} {
    echo ROSTER_PUSH $jid $name $groups $subsc $ask $category $subtype
}

proc recv_auth_result {res args} {
  global auth_result

  if {$res == "OK"} {
    set auth_result OK
  } else {
	  set auth_result ERR 
	}
}

make_gui
